#!/bin/sh
##############################################################################
#
#  Copyright (c) 2011, Doremi Labs, Inc.
#    All rights reserved.
#
#  Redistribution in source or binary forms, with or without
#  modification, are NOT permitted.
#
#    Doremi Labs, Inc.
#    1020 Chestnut St.
#    Burbank, CA 91506
#    Tel : (818) 562 1101
#    Fax : (818) 562 1109
#
# BVI: RAID status is now handled by harddiskstatus.sh
# HMO: fix to report correct status for linear and stripping RAID
# HMO: add .$$ identification in temporrary file to prevent race conflict
# HMO: change ipmi temporary filename to prevent race conflict
# HMO: add number of drives in RAID table
# HMO: process board information only once
# HMO: booting device properties added
#
##############################################################################

. /doremi/etc/init.d/functions
test -f /doremi/etc/init.d/functions || USE_FUNCTION=0

SNMP_PATH=/tmp/snmp
ALARM_PATH=/tmp/alarms
ALARM_LOGFILE=/doremi/log/alarms.log
DOREMI_NUMBER=24391
DOREMI_OID=.1.3.6.1.4.1.${DOREMI_NUMBER}
LDBCTL="/doremi/sbin/ldbctl.out"

mkdir -p "$SNMP_PATH"
mkdir -p "$ALARM_PATH"

log_alarm ()
{
	local ts=$(date +"%a %b %e %T %Y")
	echo "[$ts]: $*" >> ${ALARM_LOGFILE}
}

set_string_value ()
{
	echo "string" >  "$1.$$.tmp"
	echo "$2"     >> "$1.$$.tmp"
	mv "$1.$$.tmp" "$1"
}

set_integer_value ()
{
	echo "integer" >  "$1.$$.tmp"
	echo "$2"      >> "$1.$$.tmp"
	mv "$1.$$.tmp" "$1"
}

set_counter_value ()
{
	echo "counter" >  "$1.$$.tmp"
	echo "$2"     >> "$1.$$.tmp"
	mv "$1.$$.tmp" "$1"
}

set_gauge_value ()
{
	echo "gauge" >  "$1.$$.tmp"
	echo "$2"     >> "$1.$$.tmp"
	mv "$1.$$.tmp" "$1"
}

volt_to_mV_value ()
{
	# convert coma to dot
	local val=${1//,/.}

	# strip leading + symbol and v and V letters
	val=${val//[+vV]/}

	# bc may return decimal values
	# to remove them, use the scale variable
	# scale is only active on divisions, so we devide by 1
	echo "scale=0; $val * 1000 / 1" | bc
}

save_to_mV_string_value ()
{
	local filename="${SNMP_PATH}/$1"
	shift
	local val=$1
	shift

	local val_mV=$(volt_to_mV_value $val)
	echo "$val_mV" > "${filename}.$$.tmp"
	mv "${filename}.$$.tmp" "${filename}"
}

save_string_value ()
{
	local filename="${SNMP_PATH}/$1"

	shift
	if [ "$1" = "-" ]; then
		sed -e 's/^[ \t]*//' > "${filename}.$$.tmp"
	else
		echo "$*" > "${filename}.$$.tmp"
	fi
	mv "${filename}.$$.tmp" "${filename}"
}

save_integer_value ()
{
	local filename="${SNMP_PATH}/$1"
	local -i value

	shift
	if [ "$1" = "-" ]; then
		read value
	else
		value=$1
	fi
	echo ${value} | sed -e 's/^[ \t]*//' > "${filename}.$$.tmp"
	mv "${filename}.$$.tmp" "${filename}"
}

set_alarm_state()
{
	local KEYWORD="$1"
	local STATE="$2"

	case "$STATE" in
		normal)
			log_alarm "${KEYWORD} alarm cleared"
			test -r ${ALARM_PATH}/${KEYWORD}_error && rm -f ${ALARM_PATH}/${KEYWORD}_error
			test -r ${ALARM_PATH}/${KEYWORD}_warning && rm -f ${ALARM_PATH}/${KEYWORD}_warning
			;;
		warning)
			log_alarm "${KEYWORD} warning alarm raised"
			test -r ${ALARM_PATH}/${KEYWORD}_error && rm -f ${ALARM_PATH}/${KEYWORD}_error
			echo 1 > ${ALARM_PATH}/${KEYWORD}_warning
			;;
		error)
			log_alarm "${KEYWORD} error alarm raised"
			test -r ${ALARM_PATH}/${KEYWORD}_warning && rm -f ${ALARM_PATH}/${KEYWORD}_warning
			echo 1 > ${ALARM_PATH}/${KEYWORD}_error
			;;
		*)
			;;
	esac
}

get_alarm_state()
{
	local KEYWORD="$1"
	local STATE="normal"

	test -r ${ALARM_PATH}/${KEYWORD}_warning && STATE="warning"
	test -r ${ALARM_PATH}/${KEYWORD}_error && STATE="error"

	echo $STATE
	return 0
}

update_alarm_state()
{
	local KEYWORD="$1"
	local STATE="$2"

	if [ "$(get_alarm_state $KEYWORD)" != "$STATE" ]; then
		set_alarm_state $KEYWORD $STATE
	fi
	return 0
}

check_dlm_feature_list() {
	local dlm_feature=$1
	shift
	local dlm_value=$1
	shift

	for (( ; $# > 1 ; )); do
		local name=$1
		local value=$2
		shift 2

		# check feature value is good (case insensitive)
		if [ "$(echo $value | tr '[A-Z]' '[a-z]')" != "$(echo $dlm_value | tr '[A-Z]' '[a-z]')" ]; then
			continue
		fi

		if [ "$name" == ${dlm_feature} ]; then
			return 0
		fi
	done

	return 1
}

# returns:
#	0 : error
#	1 : available
#	2 : available with warning
#	3 : not available
check_dlm_feature() {
	local path=$1
	shift
	local warning_limit=$1
	shift
	local feature=$1
	shift
	local dlm_feature=$1
	shift
	local dlm_value=$1
	shift

	local found_feature=0
	local feature_warning=1
	local dlms=$(ls $path)
	local curr=$(date -u +%s)

	if [ -n "$dlms" ]; then
		local file
		for file in $dlms; do
			local validity_start=$(grep -a \<LicenseNotValidBefore\> ${path}/$file | awk -F \> '{print $2}' | awk -F \< '{print $1}')
			local validity_end=$(grep -a \<LicenseNotValidAfter\> ${path}/$file | awk -F \> '{print $2}' | awk -F \< '{print $1}')
			local start=$(timestamp_to_sec $validity_start)
			local end=$(timestamp_to_sec $validity_end)

			if [ ! -n "$start" ] || [ ! -n "$end" ]; then
				# Failed to parse DLM validity range
				continue
			fi

			if [ "$start" -gt "$curr" ] || [ "$curr" -gt "$end" ]; then
				# DLM not yet valid
				continue
			fi

			#                    read the <Name>/<Value> lines following <Feature>               keep <Name> and <Value> values
			local dlm_features=$(grep -a -A2 \<Feature\> ${path}/$file | grep -e Name -e Value | cut -d \< -f 2 | cut -d \> -f 2)
			if check_dlm_feature_list $dlm_feature $dlm_value $dlm_features; then
				found_feature=1
				if [ $end -gt $(( $curr + $warning_limit )) ]; then
					feature_warning=0
					break
				fi
				return 0
			fi
		done
	fi

	if [ $found_feature -eq 1 ]; then
		#Active
		if [ $feature_warning -eq 1 ]; then
			return 2
		else
			return 1
		fi
	else
		#NotActive
		return 3
	fi
}

# returns:
#	0 : error
#	1 : available
#	2 : available with warning
#	3 : not available
check_sm_feature() {
	local sm_feature=$1
	shift
	if [ -e "$SNMP_PATH/FEATURE_SM_${sm_feature}" ]; then
		# feature available
		return 1
	else
		# feature not available
		return 3
	fi
}

# PARAMS:
#	1: path to look for dlms in
#	2: amount of seconds before the DLM expires (triggers a warning)
#	3: snmp export KEYWORD (/tmp/snmp/FEATURE_XXX)
#	4: dlm feature name (optional)
#	5: dlm feature value (optional)
#	6: sm feature KEYWORD (optional) (/tmp/snmp/FEATURE_SM_XXX)
check_feature() {
	local path=$1
	shift
	local warning_limit=$1
	shift
	local feature=$1
	shift
	local dlm_feature=$1
	shift
	local dlm_value=$1
	shift
	local sm_feature=$1
	shift
	local available=1

	# check SM side of the feature is available
	if [ -n "$sm_feature" ]; then
		check_sm_feature $sm_feature
		available=$?
	fi

	if [ $available -eq 1 ]; then
		if [ -n "$dlm_feature" ]; then
			check_dlm_feature "$path" "$warning_limit" "$feature" "$dlm_feature" "$dlm_value"
			available=$?
		fi
	fi
	case "$available" in
		1)
			# feature available
			echo -n 3 > $SNMP_PATH/FEATURE_${feature}
			;;

		2)
			# feature available
			#DLM will expire in less than $warning_limit seconds
			# since no state is defined for this, leave a active
			# TODO: change to warning when state is defined in MIB
			echo -n 3 > $SNMP_PATH/FEATURE_${feature}
			;;

		3)
			# feature not available
			echo -n 4 > $SNMP_PATH/FEATURE_${feature}
			;;
		*)
	esac
}

# inverts a featuer status (required when DLM disables the feature
invert_feature() {
	local feature=$1
	shift
	local val=$(cat $SNMP_PATH/FEATURE_${feature})

	case "$val" in
		3)
			echo -n 4 > $SNMP_PATH/FEATURE_${feature}
			;;
		4)
			echo -n 3 > $SNMP_PATH/FEATURE_${feature}
			;;
		*)
			;;
	esac
}

na_feature() {
  local feature=$1
  shift
   echo -n 2 > $SNMP_PATH/FEATURE_${feature}
}

active_feature() {
  local feature=$1
  shift
   echo -n 3 > $SNMP_PATH/FEATURE_${feature}
}

check_standby_feature() {
  if [ ! -e /doremi/etc/init.d/doremi-standby.d ]; then
    echo -n 2 > $SNMP_PATH/FEATURE_standby
  elif [ ! -e /doremi/etc/standby_support ]; then
    echo -n 4 > $SNMP_PATH/FEATURE_standby
  else
    local val=$(cat /doremi/etc/standby_support)
    if [ $val -gt 0 ]; then
      echo -n 3 > $SNMP_PATH/FEATURE_standby
    else
      echo -n 4 > $SNMP_PATH/FEATURE_standby
    fi
  fi
}


remove_feature() {
  local feature=$1
  shift
   rm -f $SNMP_PATH/FEATURE_${feature}
}

inactive_feature() {
  local feature=$1
  shift
   echo -n 4 > $SNMP_PATH/FEATURE_${feature}
}

# echoes the state of the feature. State can be:
#   not_applicable: the feature does not apply to current product
#   active:         the feature applies to current product and is active
#   not_active:     the feature applies to current product and is not active
feature_state() {
  local feature=$1
  local val=$(cat $SNMP_PATH/FEATURE_${feature} 2>/dev/null)

  if [ -z "$val" ]; then
    echo "not_applicable"
  else
    case "$val" in
      3)
        echo "active"
        ;;
      4)
        echo "not_active"
        ;;
      *)
        echo "not_applicable"
        ;;
    esac
  fi
  return 0
}

feature_active() {
  local feature=$1
  if [ "$(feature_state $feature)" == "active" ]; then
    return 0
  fi

  return 1
}


check_sm_features() {
  local warning_limit=$(( 60*60*24*7 )) # 1 week
  check_feature "/doremi/dlms" $warning_limit "dolbyatmos"  ""                      ""      "dolbyatmos"
  check_feature "/doremi/dlms" $warning_limit "dual_imb"    ""                      ""      "dual_imb"    
  check_feature "/doremi/dlms" $warning_limit "auro"        "doremi.auro.enable"    "true"  "auro"   
}

check_features_sms() {
  local warning_limit=$(( 60*60*24*7 )) # 1 week
  check_feature "/doremi/dlms" $warning_limit "sensio3d"        "sensio.3d.enable"          "true"  ""
#  check_feature "/doremi/dlms" $warning_limit "closedcaption"   "doremi.cc.enable"          "true"  ""
  # reald3d does not need a dlm anymore
  #check_feature "/doremi/dlms" $warning_limit "reald3d"         "reald.3d-eq.enable"        "true"  ""
  active_feature "reald3d"
  check_feature "/doremi/dlms" $warning_limit "reald3d-sbs"     "reald.3d-sbs.enable"       "true"  ""
  # this one is a mess: dlm inverts feature availability
  check_feature "/doremi/dlms" $warning_limit "audiomark"       "doremi.audiomark.disable"  "true"  ""
  invert_feature "audiomark"
  check_feature "/doremi/dlms" $warning_limit "streamit"        "doremi.streamit.enable"    "true"  ""
  check_feature "/doremi/dlms" $warning_limit "mda"             "doremi.mda.enable"         "true"  ""
  check_feature "/doremi/dlms" $warning_limit "deghost"         "doremi.deghost.enable"     "true"  ""

  active_feature "dolby3d"
  active_feature "intermission"
  active_feature "subtitle"
  active_feature "chase"
  # do not activate NAS support on DCP2000 (Jira DCPLYR-1409)
  if [ "${v_platform}" != "dcp2000" ]; then
  	active_feature "nas"
  fi
  active_feature "closedcaption"
}

check_features_dcp2000() {
  check_features_sms
  check_sm_features
}

check_features_ims() {
  check_features_dcp2000
}

check_features_np4000() {
  local warning_limit=$(( 60*60*24*7 )) # 1 week
  
  check_features_sms
  
  active_feature "chase"

  check_sm_features
}

check_features_ims2() {
  check_features_dcp2000
}

check_features_ims3() {
  check_features_dcp2000
}

check_features_coreav() {
  local warning_limit=$(( 60*60*24*7 )) # 1 week
  check_feature "/doremi/dlms" $warning_limit "sensio3d"        "sensio.3d.enable"          "true"  ""
  check_feature "/doremi/dlms" $warning_limit "reald3d-sbs"     "reald.3d-sbs.enable"       "true"  ""
  check_feature "/doremi/dlms" $warning_limit "streamit"        "doremi.streamit.enable"    "true"  ""

  active_feature "subtitle"
}

check_features_np100() {
  check_features_coreav

  active_feature "chase"
}

check_features_le100() {
  check_features_coreav

  na_feature "chase"
}

check_features() {
 check_features_${v_platform}

 check_standby_feature
}

send_notification() {
  local event_severity="$1"
  local event_class="$2"
  local event_type="$3"
  local event_subtype="$4"
  local event_title="$5"
  local event_errorcode="$6"
  local event_desc="$7"

  # send a log
  ${LDBCTL} -s EventNotification "Severity=${event_severity}" "Class=${event_class}" \
  "EventType=${event_type}" "EventSubtype=${event_subtype}" "Title=${event_title}" \
  "ErrorCode=${event_errorcode}" "Description=${event_desc}" | \
  ${LDBCTL} -l
}
