#!/bin/sh
##############################################################################
#
#  Copyright (c) 2005, Doremi Labs, Inc.
#    All rights reserved.
#
#  Redistribution in source or binary forms, with or without
#  modification, are NOT permitted.
#
#    Doremi Labs, Inc.
#    1020 Chestnut St.
#    Burbank, CA 91506
#    Tel : (818) 562 1101
#    Fax : (818) 562 1109
#
##############################################################################

SIMULATE=0
VERBOSE=0
INSTALL=/usr/bin/install

test $SIMULATE -eq 1 && VERBOSE=1

DOREMI_X_FILES="/doremi/etc/apcupsd
/doremi/etc/apcupsd
/doremi/etc/blacklist.conf
/doremi/etc/certs/css
/doremi/etc/*.sample
/doremi/etc/cpanel.d
/doremi/etc/cron.d
/doremi/etc/cueslib/ecna.xml
/doremi/etc/cueslib/jnior.xml
/doremi/etc/cueslib/jnior2.xml
/doremi/etc/cueslib/jnior expansion module.xml
/doremi/etc/cueslib/Certainty.xml
/doremi/etc/default/*.sample
/doremi/etc/doremi-mib.mib
/doremi/etc/fontconfig/30-zh-enlarge.conf
/doremi/etc/hotplug.d
/doremi/etc/init.d
/doremi/etc/rc.d
/doremi/etc/rc.once
/doremi/etc/release.md5
/doremi/etc/upgrade
/doremi/etc/migrate.d
/doremi/etc/snmp/*.MIB
/doremi/opt/cache
/doremi/opt/bootsequence.txt
/doremi/opt/drmplaylistd/loaded-showplaylist.xml
/doremi/opt/drmplaylistd/tls.pem
/doremi/opt/drmplaylistd/tls.key
/doremi/opt/cinelister/status.xml
/doremi/opt/dbs/doremi.db
/doremi/opt/dbs/smpte.db
/doremi/opt/migrate
/doremi/opt/timestat
/doremi/opt/raidstat
/doremi/opt/smart"

if [ -z "$BACKUP_FOLDER" ]; then
  BACKUP_FOLDER=/data/migrate
fi
if [ -z "$BASE_FOLDER" ]; then
  CATEGORY=$(basename $(dirname $0))
  BASE_FOLDER=${BACKUP_FOLDER}/${CATEGORY}
fi

#
# lists files matching input patterns in first arg folder
#
safe_list_priv ()
{
  local base=$1
  shift
  
  res=$(echo "$*" | sed -e "s_[^\n]*_${base}/&_")
  ls -d $res 2>/dev/null
}

#
# lists files matching input patterns in BASE_FOLDER
#
safe_list ()
{
  local SAVEIFS=$IFS
  IFS=$(echo -en "\n\b")
  
  safe_list_priv ${BASE_FOLDER} "$*"

  IFS=$SAVEIFS
}

#
# checks if file exists in first arg folder
# before deleting.
# accepts regular expressions
#
safe_delete_priv ()
{
  local base=$1
  shift
  
  local items=$(safe_list_priv $base "$*")
  
  for item in $items; do
    if [ -e $item ]; then
      test "$VERBOSE" == "1" && echo "rm -rf $item"
      rm -rf $item
    fi
  done
}

#
# checks if file exists in BASE_FOLDER
# before deleting.
# accepts regular expressions
#
safe_delete ()
{
  local SAVEIFS=$IFS
  IFS=$(echo -en "\n\b")
  
  safe_delete_priv $BASE_FOLDER "$*"

  IFS=$SAVEIFS  
}

#
# checks if file exists on the system
# before copying
# accepts regular expressions
# copies into BASE_FOLDER
#
safe_backup ()
{
  local SAVEIFS=$IFS
  IFS=$(echo -en "\n\b")
  
  local items=$(ls -d $* 2>/dev/null)
    
  for item in $items; do
    test "$VERBOSE" == "1" && echo "mkdir -p ${BASE_FOLDER}/$(dirname $item)"
    mkdir -p ${BASE_FOLDER}/$(dirname $item)
    test "$VERBOSE" == "1" && echo "cp -a $item ${BASE_FOLDER}/$item "
    cp -a $item ${BASE_FOLDER}/$item  
  done

  IFS=$SAVEIFS  
}

#
# checks if input file is one of the sensitive files
#
is_sensitive()
{
  case $1 in
  "/etc/passwd")
    return 0
    ;;
  "/etc/shadow")
    return 0
    ;;
  esac

  return 1
}

function escape_sed_input()
{
        echo "$1" | sed -e 's/[\/&]/\\&/g'
}

#
# copies sensitive files by preserving password hashes
# and restoring correct file permissions
#
safe_restore_sensitive()
{
  bkp_pwd_file="$BASE_FOLDER/etc/passwd"
  bkp_sdw_file="$BASE_FOLDER/etc/shadow"

  tmp_pwd_file=$(tempfile -m 0640 -d /tmp/ -s tmp_restore_pwd)
  tmp_sdw_file=$(tempfile -m 0640 -d /tmp/ -s tmp_restore_sdw)

  # copy files into temporary place for processing
  cp $bkp_pwd_file $tmp_pwd_file || return 1
  cp $bkp_sdw_file $tmp_sdw_file || return 1

  test -f $tmp_pwd_file || return 1
  test -f $tmp_sdw_file || return 1

  # get list of all root privileged users on backed up file
  root_list="$(awk -F: '$3 == 0 || $4 == 0 { print $1 }' ${tmp_pwd_file})"

  # for all privileged users do not restore password - keep current system password.
  # if user is not already in the system then don't restore that user.
  for user_name in ${root_list}; do
    user_hash="$(grep -E "^${user_name}\:" /etc/shadow 2> /dev/null | cut -d: -f2 )"

    if [ -z "${user_hash}" ]; then
      sed -i -e "/^${user_name}:/d" ${tmp_pwd_file}
      sed -i -e "/^${user_name}:/d" ${tmp_sdw_file}
    else
      sed -i -e "s/^\(${user_name}:\)[^:]*/\1$(escape_sed_input ${user_hash})/" ${tmp_sdw_file}
    fi
  done

  # sanity check: make sure we never end up without a root user
  grep -q "^root:" ${tmp_pwd_file} || return 1
  grep -q "^root:" ${tmp_sdw_file} || return 1

  # backup existing system files
  cp /etc/shadow   /etc/shadow-
  cp /etc/passwd   /etc/passwd-

  # restore system files and permissions
  ${INSTALL} -o root -g root -m 0644 ${tmp_pwd_file} /etc/passwd
  ${INSTALL} -o root -g shadow -m 0640 ${tmp_sdw_file} /etc/shadow

  rm -f ${tmp_pwd_file}
  rm -f ${tmp_sdw_file}

  return 0
}

#
# checks if file exists in BASE_FOLDER
# before restoring it
# accepts regular expressions
#
safe_restore ()
{
  local SAVEIFS=$IFS
  IFS=$(echo -en "\n\b")

  has_sensitive=0
  local items=$(safe_list "$*")
  for item in $items; do
    local orig_item=$(echo ${item/${BASE_FOLDER}//} | sed -re 's@[\/]{2,}@\/@g')
    if [ "SIMULATE" == "1" ]; then
      echo "mkdir -p $(dirname $orig_item)"
      if [ -d $item ]; then
        echo "cp -a $item $(dirname $orig_item)"
      else
        if ! is_sensitive $orig_item; then
          echo "cp -a $item $orig_item"
        fi
      fi
    else
      test "$VERBOSE" == "1" && echo "mkdir -p $(dirname $orig_item)"
      mkdir -p $(dirname $orig_item)
      if [ -d $item ]; then
        test "$VERBOSE" == "1" && echo "cp -a $item $(dirname $orig_item)"
        cp -a $item $(dirname $orig_item)
      else
        test "$VERBOSE" == "1" && echo "cp -a $item $orig_item"
        if is_sensitive $orig_item; then
          has_sensitive=1
        else
          cp -a $item $orig_item
        fi
      fi

      # restore correct ownership (DCPLYR-3676)
      # NOTE: PERM_LIST is assumed to contain paths in successive descendence..
      # In the following example, the subfolder "aa" must be after "test"
      # /etc/test[root:admin]
      # /etc/test/aa[root:admin]
      #
      for perm_field in ${PERM_LIST}; do
        perm_path="$(echo "${perm_field}" | sed -rn 's/(.*)\[[a-z\:\-]*\]/\1/p')"
        perm="$(echo "${perm_field}" | sed -rn 's/.*\[([a-z\:\-]*)\]/\1/p')"
        if [[ "${perm_path}" == ${orig_item}* ]]; then
	  if [ -e ${perm_path} ]; then
            test "$VERBOSE" == "1" && echo "chown -R "${perm}" ${perm_path}"
            chown -hPR "${perm}" ${perm_path}
          fi
        fi
      done
    fi
  done

  # process sensitive files in a sensitive manner
  if [ $has_sensitive -eq 1 ]; then
    test "$VERBOSE" == "1" && echo "safe_restore_sensitive"
    safe_restore_sensitive
  fi

  IFS=$SAVEIFS  
}

#
# remove unwanted files from backup folder
# specified as first argument
#
exclude_doremi_x_file()
{
  local SAVEIFS=$IFS
  IFS=$(echo -en "\n\b")
  
  safe_delete_priv $1 "$DOREMI_X_FILES"

  IFS=$SAVEIFS  
}
